@@ -1,7 +1,6 @@
-/* $Id$
- *
+/*
  * Network Performance Meter
- * Copyright (C) 2009-2016 by Thomas Dreibholz
+ * Copyright (C) 2009-2017 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -238,7 +237,7 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
    if(awaitNetPerfMeterAcknowledge(messageReader, controlSocket,
                                    flow->getMeasurementID(),
                                    flow->getFlowID(), flow->getStreamID()) == false) {
-      perror("sctp_rev error");
+      perror("sctp_recv error");
       return(false);
    }
 
@@ -598,7 +597,7 @@ bool awaitNetPerfMeterAcknowledge(MessageReader* messageReader,
       }
       return(false);
    }
-   if(!(pfd.revents & POLLIN)) {
+   if(!(pfd.revents & (POLLIN|POLLERR))) {
       if(gOutputVerbosity >= NPFOV_CONNECTIONS) {
          std::cout << "<no answer> ";
       }
