@@ -1,8 +1,7 @@
-/* $Id$
- *
+/*
  * Network Performance Meter
  * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
- * Copyright (C) 2009-2016 by Thomas Dreibholz
+ * Copyright (C) 2009-2017 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -735,6 +734,7 @@ void FlowManager::run()
                   pollFDs[n].events  = POLLIN;
                   pollFDs[n].revents = 0;
                   FlowSet[i]->PollFDEntry = &pollFDs[n];
+                  // printf("?pollin-1: %d\n", pollFDs[n].fd);
                   n++;
                   socketSet.insert(FlowSet[i]->SocketDescriptor);
                }
@@ -753,6 +753,7 @@ void FlowManager::run()
          pollFDs[n].fd      = iterator->first;
          pollFDs[n].events  = POLLIN;
          pollFDs[n].revents = 0;
+         // printf("?pollin-2: %d\n", pollFDs[n].fd);
          unidentifiedSocketsPollFDIndex[i] = &pollFDs[n];
          n++; i++;
       }
@@ -784,10 +785,13 @@ void FlowManager::run()
             // NOTE: Release the lock here, because the FlowSet enty may belong
             //       to another stream of the socket. handleNetPerfMeterData()
             //       will find and lock the actual FlowSet entry!
-            if( (entry) && (entry->revents & POLLIN) ) {
-               // NOTE: FlowSet[i] may not be the actual Flow!
-               //       It may be another stream of the same SCTP assoc!
-               handleNetPerfMeterData(true, now, protocol, entry->fd);
+            if(entry) {
+               // printf("***pollin-1: %d REV=%x\n", entry->fd, entry->revents);
+               if(entry->revents & (POLLIN|POLLERR)) {
+                  // NOTE: FlowSet[i] may not be the actual Flow!
+                  //       It may be another stream of the same SCTP assoc!
+                  handleNetPerfMeterData(true, now, protocol, entry->fd);
+               }
             }
             FlowSet[i]->unlock();
          }
@@ -799,7 +803,8 @@ void FlowManager::run()
             for(std::map<int, int>::iterator iterator = UnidentifiedSockets.begin();
                iterator != UnidentifiedSockets.end(); iterator++) {
                assert(unidentifiedSocketsPollFDIndex[i]->fd == iterator->first);
-               if(unidentifiedSocketsPollFDIndex[i]->revents & POLLIN) {
+               if(unidentifiedSocketsPollFDIndex[i]->revents & (POLLIN|POLLERR)) {
+                  // printf("***pollin-2: %d REV=%x\n", unidentifiedSocketsPollFDIndex[i]->fd, unidentifiedSocketsPollFDIndex[i]->revents);
                   if(handleNetPerfMeterData(true, now,
                                             iterator->second,
                                             iterator->first) == 0) {
@@ -1056,10 +1061,13 @@ void Flow::deactivate(const bool asyncStop)
             // timeout.
          }
          else {
-            const int shutdownOkay = ext_shutdown(SocketDescriptor, 2);
+            // const int shutdownOkay =
+               ext_shutdown(SocketDescriptor, 2);
+            /*
             if(shutdownOkay < 0) {
-               perror("WARNING: Cannut shut association down");
+               perror("WARNING: Failed to shut down association");
             }
+            */
          }
       }
       if(!asyncStop) {
