@@ -1,7 +1,6 @@
-/* $Id$
- *
+/*
  * Network Performance Meter
- * Copyright (C) 2009-2016 by Thomas Dreibholz
+ * Copyright (C) 2009-2017 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,7 +60,7 @@ std::string format(const char* fmt, ...)
 }
 
 
-// ###### Get current timer #################################################
+// ###### Get current time stamp ############################################
 unsigned long long getMicroTime()
 {
   struct timeval tv;
@@ -1232,7 +1231,7 @@ ssize_t sctp_send_fixed(int                           sd,
    struct sctp_sndrcvinfo* sri;
    struct iovec            iov = { (char*)data, len };
    struct cmsghdr*         cmsg;
-   size_t                  cmsglen = CMSG_SPACE(sizeof(struct sctp_sndrcvinfo));
+   socklen_t               cmsglen = CMSG_SPACE(sizeof(struct sctp_sndrcvinfo));
    char                    cbuf[CMSG_SPACE(sizeof(struct sctp_sndrcvinfo))];
    struct msghdr msg = {
       NULL, 0,
