@@ -1,7 +1,6 @@
-/* $Id$
- *
+/*
  * Network Performance Meter
- * Copyright (C) 2016 by Thomas Dreibholz
+ * Copyright (C) 2016-2017 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,8 +46,8 @@
 #define IPPROTO_MPTCP IPPROTO_EGP
 
 /* FIXME: This is ugly, but currently the only way to easily get the #defines for Linux MPTCP! */
-#define TCP_MULTIPATH_DEBUG         10001   /* MPTCP DEBUG on/off    */
-#define TCP_MULTIPATH_ENABLE        26      /* MPTCP DISABLED on/off */
+#define TCP_MULTIPATH_DEBUG         10001   /* MPTCP DEBUG on/off  */
+#define TCP_MULTIPATH_ENABLE        42      /* MPTCP ENABLE on/off */
 #define TCP_MULTIPATH_ENABLE_LEGACY 10002
 #define TCP_MULTIPATH_ADD           10003
 #define TCP_MULTIPATH_REMOVE        10004
